@@ -4,6 +4,7 @@ import (
 	"context"
 	"encoding/json"
 	"flag"
+	"fmt"
 	"strconv"
 	"strings"
 	"time"
@@ -17,7 +18,7 @@ import (
 	"github.com/open-policy-agent/gatekeeper/pkg/util"
 	"github.com/pkg/errors"
 	corev1 "k8s.io/api/core/v1"
-	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
@@ -269,10 +270,10 @@ func (am *Manager) auditResources(
 		if _, ok := clusterAPIResources[gv]; !ok {
 			clusterAPIResources[gv] = make(map[string]bool)
 		}
-		for _, resource := range rl.APIResources {
-			for _, verb := range resource.Verbs {
+		for i := range rl.APIResources {
+			for _, verb := range rl.APIResources[i].Verbs {
 				if verb == "list" {
-					clusterAPIResources[gv][resource.Kind] = true
+					clusterAPIResources[gv][rl.APIResources[i].Kind] = true
 					break
 				}
 			}
@@ -361,9 +362,9 @@ func (am *Manager) auditResources(
 					continue kindsLoop
 				}
 
-				for _, obj := range objList.Items {
-					objNamespace := obj.GetNamespace()
-					isExcludedNamespace, err := am.skipExcludedNamespace(&obj)
+				for index := range objList.Items {
+					objNamespace := objList.Items[index].GetNamespace()
+					isExcludedNamespace, err := am.skipExcludedNamespace(&objList.Items[index])
 					if err != nil {
 						log.Error(err, "error while excluding namespaces")
 					}
@@ -382,7 +383,7 @@ func (am *Manager) auditResources(
 					}
 
 					augmentedObj := target.AugmentedUnstructured{
-						Object:    obj,
+						Object:    objList.Items[index],
 						Namespace: &ns,
 					}
 					resp, err := am.opa.Review(ctx, augmentedObj)
@@ -429,18 +430,16 @@ func (am *Manager) auditManagerLoop(ctx context.Context) {
 }
 
 // Start implements controller.Controller
-func (am *Manager) Start(stop <-chan struct{}) error {
+func (am *Manager) Start(ctx context.Context) error {
 	log.Info("Starting Audit Manager")
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
 	go am.auditManagerLoop(ctx)
-	<-stop
+	<-ctx.Done()
 	log.Info("Stopping audit manager workers")
 	return nil
 }
 
 func (am *Manager) ensureCRDExists(ctx context.Context) error {
-	crd := &apiextensionsv1beta1.CustomResourceDefinition{}
+	crd := &apiextensionsv1.CustomResourceDefinition{}
 	return am.client.Get(ctx, types.NamespacedName{Name: crdName}, crd)
 }
 
@@ -458,8 +457,8 @@ func (am *Manager) getAllConstraintKinds() ([]schema.GroupVersionKind, error) {
 	version := resourceGV[1]
 	// We have seen duplicate GVK entries on shifting to status client, remove them
 	unique := make(map[schema.GroupVersionKind]bool)
-	for _, i := range l.APIResources {
-		unique[schema.GroupVersionKind{Group: group, Version: version, Kind: i.Kind}] = true
+	for i := range l.APIResources {
+		unique[schema.GroupVersionKind{Group: group, Version: version, Kind: l.APIResources[i].Kind}] = true
 	}
 	var ret []schema.GroupVersionKind
 	for gvk := range unique {
@@ -558,7 +557,8 @@ func (ucloop *updateConstraintLoop) updateConstraintStatus(ctx context.Context,
 	ucloop.log.Info("updating constraint status", "constraintName", constraintName)
 	// create constraint status violations
 	var statusViolations []interface{}
-	for _, ar := range auditResults {
+	for i := range auditResults {
+		ar := &auditResults[i] // avoid large shallow copy in range loop
 		// append statusViolations for this constraint until constraintViolationsLimit has reached
 		if uint(len(statusViolations)) < *constraintViolationsLimit {
 			msg := ar.message
@@ -676,7 +676,7 @@ func (ucloop *updateConstraintLoop) update(ctx context.Context, constraintsGVKs
 		return
 	}
 
-	ucloop.log.Info("starting update constraints loop", "constraints", ucloop.uc)
+	ucloop.log.Info("starting update constraints loop", "constraints to update", fmt.Sprintf("%v", ucloop.uc))
 
 	updateLoop := func() (bool, error) {
 		for _, item := range ucloop.uc {
@@ -736,7 +736,7 @@ func (ucloop *updateConstraintLoop) update(ctx context.Context, constraintsGVKs
 		Jitter:   1,
 		Steps:    5,
 	}, updateLoop); err != nil {
-		ucloop.log.Error(err, "could not update constraint reached max retries", "remaining update constraints", ucloop.uc)
+		ucloop.log.Error(err, "could not update constraint reached max retries", "remaining update constraints", fmt.Sprintf("%v", ucloop.uc))
 	}
 }
 
